/**
 * @file	cf_file.h
 * @author	myusgun <myusgun@gmail.com>
 * @version	0.1
 *
 * @section 샘플코드
 * @code

#include "cf_file.h"

int main (void) {
	int fd = 0;
	char *name = "file.txt";
	char buffer[128] = {0x00,};

	fd = CF_File_Create (name);
	if (fd < 0) {
		// error
	}
	if (CF_File_Write (fd, "file test", 9) < 0) {
		// error
	}
	CF_File_Close (fd);

	fd = CF_File_Open (name, CF_FILE_RO);
	if (fd < 0) {
		// error
	}
	printf ("file size : %d\n", CF_File_GetSize (fd));
	if (CF_File_Read (fd, buffer, sizeof (buffer)) < 0) {
		// error
	}
	CF_File_Close (fd);

	return 0;
}

 * @endcode
 */
#ifndef __CF_FILE_H__
#define __CF_FILE_H__

#include "cf_base.h"
#include <fcntl.h>

#define CF_ERROR_FILE_OPEN				CF_ERROR_FILE - 1
#define CF_ERROR_FILE_INVALID_ARGS		CF_ERROR_FILE - 2
#define CF_ERROR_FILE_READ				CF_ERROR_FILE - 3
#define CF_ERROR_FILE_WRITE				CF_ERROR_FILE - 4
#define CF_ERROR_FILE_CREATE			CF_ERROR_FILE - 5
#define CF_ERROR_FILE_CLOSE				CF_ERROR_FILE - 6
#define CF_ERROR_FILE_GET_SIZE			CF_ERROR_FILE - 7

/** 파일 열기 옵션 플래그 */
typedef enum {
	CF_FILE_RO = O_RDONLY,	/**< 읽기 전용 */
	CF_FILE_WO = O_WRONLY,	/**< 쓰기 전용 */
	CF_FILE_WR = O_RDWR,	/**< 읽기/쓰기 */
	CF_FILE_CR = O_CREAT,	/**< 파일이 존재하지 않으면 생성 */
	CF_FILE_TR = O_TRUNC,	/**< 파일이 존재하면 비우기 */
	CF_FILE_AP = O_APPEND	/**< 파일이 존재하면 이어서 쓰기 */
} E_CF_FILE_FLAG, CF_FILE_FLAG;

#ifdef __cplusplus
extern "C" {
#endif

/**
 * 파일 열기
 *
 * @return 성공 시, 파일 디스크립터; 실패 시, 오류 코드
 *
 * @param path	파일 경로
 * @param flag	파일 열기 플래그
 *
 * @see CF_FILE_FLAG
 */
CF_EXPORT int
CF_File_Open			(const char			* path,
						 const CF_FILE_FLAG	flag);

/**
 * 파일 닫기
 *
 * @return 성공 시, CF_OK; 실패 시, 오류 코드
 *
 * @param fd 파일 디스크립터
 */
CF_EXPORT int
CF_File_Close			(const int fd);

/**
 * 파일 생성
 *
 * @return 성공 시, 파일 디스크립터; 실패 시, 오류 코드
 *
 * @param path	파일 경로
 */
CF_EXPORT int
CF_File_Create			(const char * path);

/**
 * 파일 읽기
 *
 * @return 성공 시, 읽은 바이트 수; 실패 시, 오류 코드
 *
 * @param fd	파일 디스크립터
 * @param buf	데이터를 저장할 메모리
 * @param len	데이터를 저장할 메모리의 크기
 */
CF_EXPORT int
CF_File_Read			(const int		fd,
						 void			* buf,
						 const size_t	len);

/**
 * 파일 쓰기
 *
 * @return 성공 시, CF_OK; 실패 시, 오류 코드
 *
 * @param fd	파일 디스크립터
 * @param buf	데이터가 저장된 메모리
 * @param len	쓸 데이터의 길이
 */
CF_EXPORT int
CF_File_Write			(const int		fd,
						 const void		* buf, 
						 const size_t	len);

/**
 * 파일 크기 얻기
 *
 * @return 성공 시, 파일 크기; 실패 시, 오류 코드
 *
 * @param fd	파일 디스크립터
 */
CF_EXPORT int
CF_File_GetSize			(const int fd);

#ifdef __cplusplus
}
#endif

#endif // #ifndef __CF_FILE_H__
